@@ -15,11 +15,19 @@ NULL
 #' @param    sequence        sequence string in which mutations are to be introduced.
 #'                           Accepted alphabet: \code{\{A, T, G, C, N, .\}}. Note
 #'                           that \code{-} is not accepted.
-#' @param    numMutations    number of mutations to be introduced into \code{sequence}.
+#' @param    numMutations    a whole number indicating the number of mutations to be 
+#'                           introduced into \code{sequence}, if \code{frequency=FALSE}.
+#'                           A fraction bewteen 0 and 1 indicating the mutation frequency
+#'                           if \code{frequency=TRUE}.
 #' @param    targetingModel  5-mer \link{TargetingModel} object to be used for computing 
 #'                           probabilities of mutations at each position. Defaults to
 #'                           \link{HH_S5F}.
-#'                           
+#' @param    start           Initial position in \code{sequence} where mutations can 
+#'                           be introduced. Default: 1
+#' @param    end             Last position in \code{sequence} where mutations can 
+#'                           be introduced. Default: last position (sequence length).
+#' @param    frequency       If \code{TRUE}, treat \code{numMutations} as a frequency.                           
+#' 
 #' @return   A string defining the mutated sequence.
 #' 
 #' @details
@@ -29,6 +37,10 @@ NULL
 #' Mutations are not introduced to positions in the input \code{sequence} that contain 
 #' \code{.} or \code{N}.
 #' 
+#' With \code{frequency=TRUE}, the number of mutations introduced is the \code{floor} of 
+#' the length of the sequence multiplied by the mutation frequency specified via
+#' \code{numMutations}.
+#' 
 #' @seealso  See \link{shmulateTree} for imposing mutations on a lineage tree. 
 #'           See \link{HH_S5F} and \link{MK_RS5NF} for predefined 
 #'           \link{TargetingModel} objects.
@@ -38,21 +50,61 @@ NULL
 #' sequence <- "NGATCTGACGACACGGCCGTGTATTACTGTGCGAGAGATA.TTTA"
 #' 
 #' # Simulate using the default human 5-mer targeting model
-#' shmulateSeq(sequence, numMutations=6)
+#' # Introduce 6 mutations
+#' shmulateSeq(sequence, numMutations=6, frequency=FALSE)
+#' 
+#' # Introduction 5% mutations
+#' shmulateSeq(sequence, numMutations=0.05, frequency=TRUE)
 #' 
 #' @export
-shmulateSeq <- function(sequence, numMutations, targetingModel=HH_S5F) {
+shmulateSeq <- function(sequence, numMutations, targetingModel=HH_S5F, 
+                        start=1, end=nchar(sequence), frequency=FALSE) {
     #* counts on constant variables CODON_TABLE, NUCLEOTIDES (ACTGN-.)
+
+    if (!frequency) {
+        # check if numMutations is a whole number
+        # is.wholenumber function borrowed from R's integer help
+        is.wholenumber <-
+            function(x, tol = .Machine$double.eps^0.5)  abs(x - round(x)) < tol
+        if (!is.wholenumber(numMutations)) {
+            stop("`numMutations` must be a whole number for frequency=FALSE.")
+        }
+    } else {
+        # check if numMutations if between 0 and 1
+        if (!(numMutations>=0 & numMutations<=1)) {
+            stop("`numMutations` must be a fraction between 0 and 1 for frequency=TRUE.")
+        }
+    }
     
     # Check targeting model
     if (!is(targetingModel, "TargetingModel")) {
         stop(deparse(substitute(targetingModel)), " is not a valid TargetingModel object")
     }
 
+    # Trim sequence to consider only the interval start:end
+    head_sequence <- ""
+    tail_sequence <- ""
+    seq_len <- stri_length(sequence)
+    if (start<1 | end>seq_len ) {
+        stop("`start` must be >= 1 and `end` must be <= sequence length")
+    } else {
+        head_sequence <- stri_sub(str=sequence, from=0, to=start-1)
+        tail_sequence <-  stri_sub(str=sequence, from=end+1, to=seq_len)
+        sequence <- stri_sub(str=sequence, from=start, to=end)
+    }
+    
     # Trim sequence to last codon (getCodonPos from MutationProfiling.R)
     if(getCodonPos(stri_length(sequence))[3] > stri_length(sequence)) {
+        warning("Trimming sequence to last codon")
         sim_seq <- stri_sub(str=sequence, from=1, 
                             to=getCodonPos(stri_length(sequence))[1]-1)
+        # Add removed chars to tail_sequence
+        tail_sequence <- paste0(
+            stri_sub(str=sequence,
+                     from=getCodonPos(stri_length(sequence))[1], 
+                     to=stri_length(sequence)),
+                     tail_sequence)
+        
     } else {
         sim_seq <- sequence
     }
@@ -60,6 +112,16 @@ shmulateSeq <- function(sequence, numMutations, targetingModel=HH_S5F) {
     sim_leng <- stri_length(sim_seq)
     stopifnot((sim_leng %% 3)==0)
     
+    # if specifying mutation frequency instead of count, 
+    # get corresponding mutation count based on sequence length
+    if (frequency) {
+        numMutations <- floor(sim_leng*numMutations)
+    }
+    
+    if (numMutations > sim_leng) {
+        stop("Number of mutations specified is larger than the length of the sequence.")
+    }
+    
     # Calculate possible mutations (given codon table)
     mutation_types <- computeMutationTypes(sim_seq)
     
@@ -75,13 +137,13 @@ shmulateSeq <- function(sequence, numMutations, targetingModel=HH_S5F) {
     # set NA to 0
     targeting[is.na(targeting)] <- 0 
     # Make probability of stop codon 0
-    targeting[mutation_types=="Stop"] <- 0
+    targeting[mutation_types=="stop"] <- 0
     
     # Initialize counters
     total_muts <- 0
     positions <- numeric(numMutations)
     
-    while(total_muts < numMutations) {
+    while (total_muts < numMutations) {
         # Get position to mutate and update counters
         mutpos <- sampleMut(sim_leng, targeting, positions)
         total_muts <- total_muts + 1
@@ -89,7 +151,8 @@ shmulateSeq <- function(sequence, numMutations, targetingModel=HH_S5F) {
         
         # Implement mutation in simulation sequence
         mut_nuc <- 4 - (4*mutpos$pos - mutpos$mut)
-        stri_sub(str=sim_seq, from=mutpos$pos, to=mutpos$pos) <- NUCLEOTIDES[mut_nuc]
+        # stri_sub(str=sim_seq, from=mutpos$pos, to=mutpos$pos) <- NUCLEOTIDES[mut_nuc]
+        sim_seq <- stri_sub_replace(str=sim_seq, from=mutpos$pos, to=mutpos$pos, value=NUCLEOTIDES[mut_nuc]) 
         
         # Update targeting
         lower <- max(mutpos$pos-4, 1)
@@ -107,12 +170,14 @@ shmulateSeq <- function(sequence, numMutations, targetingModel=HH_S5F) {
                                                                        from = lower, 
                                                                        to = upper))
         # Make probability of stop codon 0
-        if(any(mutation_types[, lower:upper]=="Stop", na.rm=T)) {
-            targeting[, lower:upper][mutation_types[, lower:upper]=="Stop"] <- 0
+        if (any(mutation_types[, lower:upper]=="stop", na.rm=T)) {
+            targeting[, lower:upper][mutation_types[, lower:upper]=="stop"] <- 0
         }
     }
     # sanity check: length of sim_seq should remain unchanged after simulation
     stopifnot(sim_leng==stri_length(sim_seq))
+    # Add back head and tail sequences
+    sim_seq <- paste0(head_sequence, sim_seq, tail_sequence)
     return(sim_seq)
 }
 
@@ -142,13 +207,16 @@ shmulateSeq <- function(sequence, numMutations, targetingModel=HH_S5F) {
 #'                           number of mutations to the immediate offspring nodes of the 
 #'                           MRCA. Requires a value between 0 and 1. If \code{NULL} then 
 #'                           edge weights are unmodified from the input \code{graph}.
-#'
+#' @param   start            Initial position in \code{sequence} where mutations can 
+#'                           be introduced. Default: 1
+#' @param   end              Last position in \code{sequence} where mutations can 
+#'                           be introduced. Default: last position (sequence length).                           
 #' @return   A \code{data.frame} of simulated sequences with columns:
 #'           \itemize{
-#'             \item \code{NAME}:      name of the corresponding node in the input 
+#'             \item \code{name}:      name of the corresponding node in the input 
 #'                                     \code{graph}.  
-#'             \item \code{SEQUENCE}:  mutated sequence.
-#'             \item \code{DISTANCE}:  Hamming distance of the mutated sequence from 
+#'             \item \code{sequence}:  mutated sequence.
+#'             \item \code{distance}:  Hamming distance of the mutated sequence from 
 #'                                     the seed \code{sequence}.
 #'           }
 #' 
@@ -169,11 +237,12 @@ shmulateSeq <- function(sequence, numMutations, targetingModel=HH_S5F) {
 #' # Exclude nodes without a sample identifier
 #' # Add 20% mutation rate to the immediate offsprings of the MRCA
 #' shmulateTree(sequence, graph, targetingModel=MK_RS5NF,
-#'              field="SAMPLE", exclude=NA, junctionWeight=0.2)
+#'              field="sample_id", exclude=NA, junctionWeight=0.2)
 #'  
 #' @export
 shmulateTree <- function(sequence, graph, targetingModel=HH_S5F,
-                         field=NULL, exclude=NULL, junctionWeight=NULL) {
+                         field=NULL, exclude=NULL, junctionWeight=NULL,
+                         start=1, end=nchar(sequence)) {
     ## DEBUG
     # targetingModel=HH_S5F; field=NULL; exclude=NULL; junctionWeight=NULL
     
@@ -199,25 +268,25 @@ shmulateTree <- function(sequence, graph, targetingModel=HH_S5F,
     # Create data.frame to hold simulated sequences
     # this will include a row for Germline
     sim_tree <- data.frame(matrix(NA, ncol=3, nrow=length(V(graph)),
-                           dimnames=list(NULL, c("NAME", "SEQUENCE", "DISTANCE"))))
-    sim_tree$NAME <- vertex_attr(graph, name="name")
+                           dimnames=list(NULL, c("name", "sequence", "distance"))))
+    sim_tree$name <- vertex_attr(graph, name="name")
     
     # remove row for Germline
-    sim_tree <- sim_tree[-which(sim_tree$NAME=="Germline"), ]
+    sim_tree <- sim_tree[-which(sim_tree$name=="Germline"), ]
         
-    parent_nodes <- mrca_df$NAME[1]
+    parent_nodes <- mrca_df$name[1]
     nchild <- sum(adj[parent_nodes, ] > 0)
     
-    sim_tree$SEQUENCE[which(sim_tree$NAME==parent_nodes)] <- sequence
-    sim_tree$DISTANCE[which(sim_tree$NAME==parent_nodes)] <- 0
+    sim_tree$sequence[which(sim_tree$name==parent_nodes)] <- sequence
+    sim_tree$distance[which(sim_tree$name==parent_nodes)] <- 0
     
     # Add mutations to the immediate offsprings of the MRCA
     # Number of mutations added is proportional to fraction of sequence in junction
-    if(!is.null(junctionWeight)) {
+    if (!is.null(junctionWeight)) {
         adj[parent_nodes, ] <- round(adj[parent_nodes, ] * (1 + junctionWeight))
     }
     
-    while(nchild > 0) {
+    while (nchild > 0) {
         new_parents <- c()
         # Loop through parent-children combos
         for(p in parent_nodes) {
@@ -226,13 +295,14 @@ shmulateTree <- function(sequence, graph, targetingModel=HH_S5F,
                 # Add child to new parents
                 new_parents <- union(new_parents, ch)
                 # Simulate sequence for that edge
-                seq <- shmulateSeq(sequence=sim_tree$SEQUENCE[sim_tree$NAME == p], 
+                seq <- shmulateSeq(sequence=sim_tree$sequence[sim_tree$name == p], 
                                    numMutations=adj[p, ch],
-                                   targetingModel=targetingModel)
+                                   targetingModel=targetingModel,
+                                   start=start, end=end)
                 # Update output data.frame
-                chRowIdx = which(sim_tree$NAME==ch)
-                sim_tree$SEQUENCE[chRowIdx] <- seq
-                sim_tree$DISTANCE[chRowIdx] <- adj[p, ch]
+                chRowIdx = which(sim_tree$name==ch)
+                sim_tree$sequence[chRowIdx] <- seq
+                sim_tree$distance[chRowIdx] <- adj[p, ch]
             }
         }
         
@@ -242,11 +312,11 @@ shmulateTree <- function(sequence, graph, targetingModel=HH_S5F,
     }
     
     # Remove sequences that are to be excluded
-    sim_tree <- sim_tree[!(sim_tree$NAME %in% skip_names), ]
+    sim_tree <- sim_tree[!(sim_tree$name %in% skip_names), ]
     # Remove NAs
     # e.g. if node B is an offspring of node A, and node A has been excluded
-    # then node B will have $SEQUENCE and $DISTANCE of NAs
-    sim_tree <- sim_tree[!is.na(sim_tree$SEQUENCE), ]
+    # then node B will have $sequence and $distance of NAs
+    sim_tree <- sim_tree[!is.na(sim_tree$sequence), ]
     
     rownames(sim_tree) <- NULL
     
@@ -272,6 +342,13 @@ computeMutationTypes <- function(inputSeq){
     try(if( (leng_seq %%3 !=0) ) stop("length of input sequence must be a multiple of 3"))
     
     codons <- sapply(seq(1, leng_seq, by=3), function(x) {substr(inputSeq,x,x+2)})
+    unrecognized_codons <- codons[!codons %in% colnames(CODON_TABLE)]
+    if (length(unrecognized_codons)>0) {
+        if (all(grepl("^[[:lower:]]+$", unrecognized_codons))) {
+            warning("shazam is case sensitive")
+        }
+        stop("Unrecognized codons found :\n", paste(unrecognized_codons, collapse="\n"))
+    }
     mut_types <- matrix(unlist(CODON_TABLE[, codons]), ncol=leng_seq, nrow=4, byrow=F)
     dimnames(mut_types) <-  list(NUCLEOTIDES[1:4], 1:leng_seq)
     return(mut_types)
@@ -288,8 +365,11 @@ computeMutationTypes <- function(inputSeq){
 # @param   targeting  probabilities of each position in the sequence being mutated
 # @param   positions  vector of positions which have already been mutated
 #
-# @return   A \code{list} of position being mutated and updated vector of mutated positions.
+# @return   A \code{list} of mutation and position being mutated.
 sampleMut <- function(sim_leng, targeting, positions) {
+    if (length(positions) > sim_leng ) {
+        stop("The vector of positions is longer than the length of the sequence.")
+    }
     pos <- 0
     # Sample mutations until new position is selected
     while (pos %in% positions) {
